/*******************************************************************************
  LCDemoCascadedDevices
  NoiascaLedControl

  This is a demo Sketch to show upwards compatibility to Eberhard Fahles LedControl Library

  credits to
  name=LedControl.cpp
  author=Eberhard Fahle
  version=1.0.3
  adopted by  noiasca
  Sketch Version 2020-03-08
 *******************************************************************************/
 
//We always have to include the library
#include "NoiascaLedControl.h"

/*
 Now we need a LedControl to work with.
 ***** These pin numbers will probably not work with your hardware *****
 */
const uint8_t LEDCS_PIN = 8;           // 8 LED CS or LOAD -  8 CS
const uint8_t LEDCLK_PIN = 13;         // 7 LED Clock      - 13 CLK on UNO/NANO
const uint8_t LEDDATA_PIN = 11;        // 9 LED DATA IN    - 11 MOSI on UNO/NANO
const uint8_t LED_MODULES = 2;     

LedControl lc = LedControl  (LEDDATA_PIN, LEDCLK_PIN, LEDCS_PIN, LED_MODULES);  // Software Bitbang - use this if you can't use Hardware SPI

/* we always wait a bit between updates of the display */
const unsigned long delaytime=500;

/* 
 This time we have more than one device. 
 But all of them have to be initialized 
 individually.
 */
void setup() {
  /* The MAX72XX needs to initialize hardware */
  lc.begin();
  //we have already set the number of devices when we created the LedControl
  int devices=lc.getDeviceCount();
  //we have to init all devices in a loop
  for(int address=0;address<devices;address++) {
    /*The MAX72XX is in power-saving mode on startup*/
    lc.shutdown(address,false);
    /* Set the brightness to a medium values */
    lc.setIntensity(address,8);
    /* and clear the display */
    lc.clearDisplay(address);
  }
}

void loop() { 
  //read the number cascaded devices
  int devices=lc.getDeviceCount();
  
  //we have to init all devices in a loop
  for(int row=0;row<8;row++) {
    for(int col=0;col<8;col++) {
      for(int address=0;address<devices;address++) {
        delay(delaytime);
        lc.setLed(address,row,col,true);
        delay(delaytime);
        lc.setLed(address,row,col,false);
      }
    }
  }
}
