/*
  Chronometer with hundredths of a second

  start a chronometer with the start button
  stop the chronometer with the stop button

  inspired by: https://forum.arduino.cc/t/sos-really-simple-question-almost-a-shame-to-ask/875082

  noiasca
  2022-01-09
  
*/

// Variant for Software Bitbang
//#include "NoiascaLedControl.h"                 // download library from https://werner.rothschopf.net/201904_arduino_ledcontrol_max7219.htm
//LedControl lc = LedControl(12, 11, 10, 1);

// Variant for HW-SPI
#include <SPI.h>                                 // needed for HW SPI
#include "NoiascaLedControlSpi.h"                // download from https://werner.rothschopf.net/201904_arduino_ledcontrol_max7219.htm
LedControlSpi lc = LedControlSpi(8, 2);          // CS = 8, 2 modules connected to HW SPI 
const byte startButton = 2;
const byte stopButton = 3;                       // Setup our button to pin assignments
bool running = false;                            // Setup our flags
unsigned long startTime;
unsigned long endTime;

void setup()
{
  Serial.begin(115200);
  Serial.println(F("\nchronometer"));
  SPI.begin();  // start SPI
  // Remember: When you use SPI on a AVR use the SS pin  (= 10 on UNO/NANO, 53 on MEGA) only as OUTPUT 
  // See https://www.arduino.cc/en/reference/SPI --> "Note about Slave Select (SS) pin on AVR boards"
  pinMode(startButton, INPUT_PULLUP);  // I suggest use both buttons as active LOW - connect to ground
  pinMode(stopButton, INPUT_PULLUP);   
  lc.shutdown(0, false);               // Setup & initialise the LCD display
  lc.setIntensity(0, 12);              // sets brightness (0~15 possible values)
  lc.clearDisplay(0);                  // clear screen
  lc.print("HELLO");                   
  delay(1000);                         
  updateDisplay();                     // Execute display data subroutine
}

void loop()
{
  if (running == false && digitalRead(startButton) == LOW) // start the chronometer
  {
    startTime = millis();
    Serial.println(F("started"));
    running = true;
  }
  if (running == true && digitalRead(stopButton) == LOW)   // stop the chronometer
  {
    Serial.println(F("stopped"));
    endTime = millis();
    running = false;
    updateDisplay();
  }
  // periodically update the display:
  static unsigned long previousMillis;
  if (running == true && millis() - previousMillis > 3)    // just an intervall to see the time "running"
  {
    previousMillis = millis();
    updateDisplay();
  }
}

void updateDisplay()
{
  unsigned long durance = millis() - startTime;  // durance in ms
  if (running == false)
  {
    durance = endTime;       // if the timer is stopped, use the previous stored endTime
  }
  Serial.println(durance);
  lc.setCursor(0, 0);
  int fraction; // two byte due to hundrets
  fraction = ((durance) / 1000UL / 60 / 60 ) % 100;        // hh
  if (fraction < 10) lc.print("0");                        
  lc.print(fraction);                                      
  lc.print(".");                                           
  fraction = ((durance) /1000UL / 60) % 60;                // mm
  if (fraction < 10) lc.print("0");                        
  lc.print(fraction);                                      
  lc.print(".");                                           
  fraction = ((durance) / 1000UL) % 60;                    // ss
  if (fraction < 10) lc.print("0");
  lc.print(fraction);
  lc.print(".");
  if (fraction < 10) lc.print("0");
  fraction = ((durance) / 10) % 100;
  lc.print(fraction); 
}
