/*******************************************************************************
  HelloWorld with
  NoiascaLedControl

  This is a demo Sketch to show up the new print methods of
  the extended LedControl Library

  name=LCDemoNoiascaHelloWorld
  version=1.0.2
  author=noiasca

  adopted by  noiasca
  Sketch Version 2022-02-28
 *******************************************************************************/

//We always have to include the library
#include "NoiascaLedControl.h"

/*
  Now we need a LedControl to work with.
 ***** These pin numbers will probably not work with your hardware *****
  Here we have only two MAX72XX.
*/
const uint8_t LEDCS_PIN = 8;           // 8 LED CS or LOAD -  8 CS
const uint8_t LEDCLK_PIN = 13;         // 13 CLK on UNO/NANO              (52 on MEGA)
const uint8_t LEDDATA_PIN = 11;        // 11 MOSI on UNO/NANO LED DATA IN (51 on MEGA)
const uint8_t LED_MODULES = 1;     

LedControl lc = LedControl (LEDDATA_PIN, LEDCLK_PIN, LEDCS_PIN, LED_MODULES);  // Software Bitbang - use this if you can't use Hardware SPI

/* we always wait a bit between updates of the display */
const unsigned long delaytime = 250;

/* we wait longer between the demos */
const unsigned long delaylong = 5000;

void setup() {
  /* Open the Serial Monitor to know what the program is doing */
  Serial.begin(115200);
  Serial.println(F("Noiasca Led Control HelloWorld"));
  /* The MAX72XX needs to initialize hardware */
  lc.begin();
  /*
    The MAX72XX is in power-saving mode on startup,
    we have to do a wakeup call
  */
  for (byte i = 0; i < LED_MODULES; i++)
  {
    lc.shutdown(i, false);
    /* Set the brightness to a medium value, possible is 0-15 */
    lc.setIntensity(i, 8);
  }
  /* and clear all displays */
  lc.clearDisplay();
}


/*
  This function will display the characters for the
  word "Arduino"
*/
void writeArduinoOn7Segment() {
  Serial.println(F("Print \"Arduino\" to LED Display"));
  lc.clearDisplay();
  lc.print("Arduino");
  delay(delaylong);
}


/*
  This function will display different characters/words
*/
void printDemo() {
  lc.clearDisplay();
  Serial.println(F("Print one character"));
  lc.print("A");
  delay(delaylong);

  lc.clearDisplay();
  Serial.println(F("Print more characters/a word: Arduino"));
  lc.print("Arduino");
  delay(delaylong);

  lc.clearDisplay();
  Serial.println(F("Print dots"));
  lc.print("So.");
  lc.print(" Cool.");
  delay(delaylong);

  lc.clearDisplay();
  Serial.println(F("Printing multiple dots needs a hack"));
  Serial.println(F("... will not work"));
  lc.print("... no");
  delay(delaylong);

  lc.clearDisplay();
  Serial.println(F(". . . . will work"));  // add a blank before consecutive dots
  lc.print(". . . . YES");
  delay(delaylong);

  lc.clearDisplay();
  Serial.println(F("Print at device 0, position 2. 0/0 is on the left. Remember: setDigit starts counting at 7!"));
  lc.setCursor(0, 2);
  lc.print("2pos");
  delay(delaylong);
}


/*
  This function will display output on multiple IC's
*/
void printMultiple()
{
#if LED_MODULES >1
  lc.clearDisplay();
  Serial.println(F("If you have more modules you can use setCursor to switch to the second MAX7219 IC"));
  lc.setCursor(1, 0);
  lc.print("2nd IC");
  delay(delaylong);

  lc.clearDisplay();
  Serial.println(F("If you have more modules you can use setCursor to set the absolute position"));
  lc.setCursor(11);
  lc.print("11POS");
  delay(delaylong);
  lc.clearDisplay();
#endif
}


/*
  This function will display different numbers on the display
*/
void printNumbers() {
  lc.clearDisplay();
  Serial.println(F("Print an integer"));
  int myint = 1234;
  lc.print(myint);
  delay(delaylong);

  lc.clearDisplay();
  Serial.println(F("Print a negative integer"));
  myint = -5678;
  lc.print(myint);
  delay(delaylong);

  lc.clearDisplay();
  Serial.println(F("Print a negative float. The decimal point will be added to the number!"));
  float myfloat = -12.34;
  lc.print(myfloat);
  delay(delaylong);
}


/*
  This function will scroll all the hexa-decimal
  numbers and letters on the display. You will need at least
  four 7-Segment digits. otherwise it won't really look that good.
*/
void scrollDigits() {
  lc.clearDisplay();
  Serial.println(F("The Character Set"));
  Serial.println(F("Character 0-10 are the numbers and the degrees sign"));
  // we use the original function
  for (int i = 0; i < 10; i++) {
    lc.setDigit(0, 3, i, false);
    lc.setDigit(0, 2, i + 1, false);
    lc.setDigit(0, 1, i + 2, false);
    lc.setDigit(0, 0, i + 3, false);
    delay(delaytime);
  }

  Serial.println(F("From character 37-127 we try to cover as many characters as possible."));
  Serial.println(F("Characters K W X are not possible and replaced with a blank. V is replaced by U. Sometimes there needs a mix of small and capital letters."));
  lc.clearDisplay();
  for (int i = 37; i < 128 - 4; i++) {
    Serial.print(char(i));
    lc.setCursor(0, 4);
    lc.print(char(i));
    lc.print(char(i + 1));
    lc.print(char(i + 2));
    lc.print(char(i + 3));
    delay(delaytime);
  }
  Serial.print(char(125));
  Serial.print(char(126));
  Serial.print(char(127));
  Serial.println();
  delay(delaylong);
}

/*
  This function shows, #
  what happens if you print more than 8 characters
  to one device/adress
*/

void showEndOfDevice()
{
  // default setting is: NEXT_DEVICE
  Serial.println(F("NEXT_DEVICE: At the End of Device (after 8 digits) we use the next available Device.\nAfter the last device go back to Device 0: NEXT_DEVICE (default)"));
  lc.clearDisplay(); // will also set the cursor to positon 0 0
  //          0123456701234567
  lc.print(F("THIS IS TO LONG"));
  delay(delaylong);

  Serial.println(F("End of device: THIS_DEVICE\n - wraps around within one device (after 8 digits) - you might miss information (indeed you don't miss it - it is just to fast, that you could read it)"));
  lc.clearDisplay();
  lc.setEndOfDevice(THIS_DEVICE);
  lc.print(F("THIS IS TO LONG"));
  delay(delaylong);

  Serial.println(F("The option FIRST_DEVICE is valid\n - but nearly all usecases can be covered with NEXT_DEVICE"));
  lc.clearDisplay();
  lc.setEndOfDevice(FIRST_DEVICE);
  lc.print(F("THIS IS TO LONG"));
  delay(delaylong);

  // now let's switch back to the default value:
  lc.setEndOfDevice(NEXT_DEVICE);
}

/*
  The library supports linefeeds = switch to next device according to the EndOfDevice setting.
*/

void showLinefeed()
{
  Serial.println(F("Show Linefeed"));
  lc.clearDisplay();
  Serial.println(F("- with println"));  //println will send a carriage return AND a linefeed - only the linefeed will be used - the carriage return will be ignored
  lc.println(F("LINE0"));
  delay(delaylong);
  Serial.println(F("- with Escape sequence \\n"));
  lc.print(F("123456\n"));
  delay(delaylong);
  lc.println(F("1234567"));
  delay(delaylong);
  Serial.println(F("Be carefull when printing 8 digits: the eighth digit will cause a linefeed - println will cause a second linefeed!"));
  lc.println(F("12345678"));
  delay(delaylong);
  lc.println(F("abcd"));
  delay(delaylong);
}

void loop() {
  writeArduinoOn7Segment();
  printDemo();
  printNumbers();
  printMultiple();
  scrollDigits();
  showEndOfDevice();
  showLinefeed();
}
