/*******************************************************************************
  LCDemoNoiasca Inversed
  NoiascaLedControl

  This is an example how to invers the display in case you have connected 
  the digits of your display in the order 01234567 instead of 76543210

  This example is based on the request of 
  https://forum.arduino.cc/index.php?topic=717187.0
  
  by noiasca
  Sketch Version 2022-01-09
 *******************************************************************************/

#include "NoiascaLedControl.h"         // include library, download from https://werner.rothschopf.net/201904_arduino_ledcontrol_max7219.htm
const uint8_t LEDCS_PIN = 8;           // 8 LED CS or LOAD -  8 CS
const uint8_t LEDCLK_PIN = 7;          // 7 LED Clock      - 13 CLK  on UNO/NANO - 52 on MEGA
const uint8_t LEDDATA_PIN = 9;         // 9 LED DATA IN    - 11 MOSI on UNO/NANO - 51 on MEGA
const uint8_t LED_MODULES = 1;

// define a new class based on LedControl
// and override the write method
class LedControlInverse : public LedControl {
  public:
    LedControlInverse(uint8_t dataPin, uint8_t clkPin, uint8_t csPin, uint8_t numDevices = 1) :    // the constructor for the new class
      LedControl(dataPin, clkPin, csPin, numDevices)                                               // you have to init the class you have inherited from
    {};

    //patch the write mehtod (used for print, println) to display in reversed order
    size_t write(uint8_t value) override{
      Serial.println(value);
      uint8_t addr = currentPosition / 8;
      uint8_t digit = (currentPosition % 8); 
      if (value == 10)                                     // \n linefeed
      {
        handleEndOfDevice(addr);
      }
      else if (value == 13)                                // \r carriage
      {
        // don't print carriage return
      }
      else if (value == '.' || value == ',')
      {
        uint8_t lastAddr = lastPosition / 8;
        uint8_t lastDigit = lastPosition % 8;
        setRow(lastAddr, lastDigit, status[lastAddr * 8 + lastDigit] | 0b10000000);      // add decimal point to the last printed digit
      }
      else
      {
        setChar(addr, digit, value, 0);
        lastPosition = currentPosition;
        currentPosition++;
        if (digit > 7) handleEndOfDevice(addr);
      }
      return 1;    // assume success
    }
};

// now you can use the class LedControlInverse for your lc object:
LedControlInverse lc = LedControlInverse  (LEDDATA_PIN, LEDCLK_PIN, LEDCS_PIN, LED_MODULES);       // Software Bitbang - use this if you can't use Hardware SPI

void setup() {
  lc.begin();                /* The MAX72XX needs to initialize hardware */
  lc.shutdown(0, false);     /* The MAX72XX is in power-saving mode on startup, do a wakeup call */
  lc.setIntensity(0, 8);     /* Set the brightness to a medium values */
  lc.clearDisplay(0);        /* and clear the display */
  
  lc.print("76.543210");     // just a demo print out
}

void loop() {
}
