/*******************************************************************************
  LCDemo7Segment - Regressiontest
  NoiascaLedControl

  This is a demo for my Regression testing

  credits to
  LedControl by Eberhard Fahle

  adopted by  noiasca
  Sketch Version 2022-01-09
 *******************************************************************************/

#define USE_SPI 1                    // switch between Software and Hardware SPI

#ifdef USE_SPI
//#include <SPI.h>                       // needed for HW SPI
#include "NoiascaLedControlSpi.h"      // download from https://werner.rothschopf.net/201904_arduino_ledcontrol_max7219.htm
/*
  Now we need a LedControl to work with.
 ***** These pin numbers will probably not work with your hardware *****
  We have only a single MAX72XX.
*/
const uint8_t LEDCS_PIN = 8;           // 8 LED CS or LOAD -  8 CS
// 7 LED Clock      - 13 CLK on UNO/NANO - 52 on MEGA
// 9 LED DATA IN    - 11 MOSI on UNO/NANO - 51 on MEGA
const uint8_t LED_MODULES = 1;
LedControlSpi lc = LedControlSpi  (LEDCS_PIN, LED_MODULES); // Hardware SPI (UNO/NANO: 13 CLK, 11 DATA_IN/MOSI)
#else
#include "NoiascaLedControl.h"        // download from https://werner.rothschopf.net/201904_arduino_ledcontrol_max7219.htm
/*
  Now we need a LedControl to work with.
 ***** These pin numbers will probably not work with your hardware *****
  We have only a single MAX72XX.
*/
const uint8_t LEDCS_PIN = 8;           // 8 LED CS or LOAD -  8 CS
const uint8_t LEDCLK_PIN = SCK;        // 7 LED Clock      - 13 CLK on UNO/NANO  - 52 on MEGA
const uint8_t LEDDATA_PIN = MOSI;      // 9 LED DATA IN    - 11 MOSI on UNO/NANO - 51 on MEGA
const uint8_t LED_MODULES = 1;
LedControl lc = LedControl  (LEDDATA_PIN, LEDCLK_PIN, LEDCS_PIN, LED_MODULES);  // Software Bitbang - use this if you can't use Hardware SPI
#endif
/* we always wait a bit between updates of the display */
const unsigned long delaytime = 250;

void setup() {
  Serial.begin(115200);
  Serial.println(F("LED Control 7 Segment Regression Test"));
#ifdef USE_SPI
  SPI.begin();  // start SPI
  // Remember: When you use SPI on a AVR use the SS pin  (= 10 on UNO/NANO, 53 on MEGA) only as OUTPUT 
  // See https://www.arduino.cc/en/reference/SPI --> "Note about Slave Select (SS) pin on AVR boards"
#endif
  /* The MAX72XX needs to initialize hardware */
  lc.begin();
  /*
    The MAX72XX is in power-saving mode on startup,
    we have to do a wakeup call
  */
  lc.shutdown(0, false);
  /* Set the brightness to a medium values */
  lc.setIntensity(0, 8);
  /* and clear the display */
  lc.clearDisplay(0);
}


/*
  This method will display the characters for the
  word "Arduino" one after the other on digit 0.
*/
void writeArduinoOn7Segment() {
  lc.setChar(0, 0, 'a', false);
  delay(delaytime);
  lc.setRow(0, 0, 0x05);
  delay(delaytime);
  lc.setChar(0, 0, 'd', false);
  delay(delaytime);
  lc.setRow(0, 0, 0x1c);
  delay(delaytime);
  lc.setRow(0, 0, B00010000);
  delay(delaytime);
  lc.setRow(0, 0, 0x15);
  delay(delaytime);
  lc.setRow(0, 0, 0x1D);
  delay(delaytime);
  lc.clearDisplay(0);
  delay(delaytime);
}

/*
  This method will scroll all the hexa-decimal
  numbers and letters on the display. You will need at least
  four 7-Segment digits. otherwise it won't really look that good.
*/
void scrollDigits() {
  for (int i = 0; i < 13 ; i++) {
    lc.setDigit(0, 3, i, false);
    lc.setDigit(0, 2, i + 1, false);
    lc.setDigit(0, 1, i + 2, false);
    lc.setDigit(0, 0, i + 3, false);
    delay(delaytime);
  }
  lc.clearDisplay(0);
  delay(delaytime);
}

void loop() {
  writeArduinoOn7Segment();
  scrollDigits();
}
