/*
 *    NoiascaLedControlSpi.cpp - A library for controling LEDss with a MAX7219/MAX7221
 *    
 *    noiasca
 *    https://werner.rothschopf.net/201904_arduino_ledcontrol_max7219.htm
 *
 *    Permission is hereby granted, free of charge, to any person
 *    obtaining a copy of this software and associated documentation
 *    files (the "Software"), to deal in the Software without
 *    restriction, including without limitation the rights to use,
 *    copy, modify, merge, publish, distribute, sublicense, and/or sell
 *    copies of the Software, and to permit persons to whom the
 *    Software is furnished to do so, subject to the following
 *    conditions:
 * 
 *    This permission notice shall be included in all copies or 
 *    substantial portions of the Software.
 * 
 *    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 *    EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 *    OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 *    NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 *    HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 *    WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 *    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 *    OTHER DEALINGS IN THE SOFTWARE.
 
 *    Version control see .h
 */

#include <NoiascaLedControlSpi.h>
/*
//the opcodes for the MAX7221 and MAX7219 - MISSING final delete in 1.2.5 or higher
#define OP_NOOP   0
#define OP_DIGIT0 1
#define OP_DIGIT1 2
#define OP_DIGIT2 3
#define OP_DIGIT3 4
#define OP_DIGIT4 5
#define OP_DIGIT5 6
#define OP_DIGIT6 7
#define OP_DIGIT7 8
#define OP_DECODEMODE  9
#define OP_INTENSITY   10
#define OP_SCANLIMIT   11
#define OP_SHUTDOWN    12
#define OP_DISPLAYTEST 15
*/

LedControlSpi::LedControlSpi(uint8_t csPin, uint8_t numDevices) :
    LedControl_base(numDevices),
    csPin(csPin)
    {}

void LedControlSpi::begin(){
  if (!isInitialized)        // backward compatibility: method could be called from first shutdown method also
  {
    pinMode(csPin, OUTPUT);
    digitalWrite(csPin, HIGH);
	  //SPI.begin();                          // must be called in the user sketch!
    if(maxDevices > 8) maxDevices = 8;
    memset(status, 0, sizeof(status));
    isInitialized = true;
  }
  for(uint8_t i = 0; i < maxDevices; i++) {
    spiTransfer(i, OP_DISPLAYTEST, 0);
    //scanlimit is set to max on startup
    setScanLimit(i, 7);
    //decode is done in source
    spiTransfer(i, OP_DECODEMODE, 0);
    clearDisplay(i);
    //we go into shutdown-mode on startup
    shutdown(i, true);
  }
}

// change noiasca: low level shift out in software or hardware-SPI
void LedControlSpi::spiTransfer(uint8_t addr, volatile uint8_t opcode, volatile uint8_t data) {
  //Create an array with the data to shift out
  uint8_t offset = addr * 2;
  uint8_t maxbytes = maxDevices * 2;
  memset(spidata, 0, sizeof(spidata));
  //put our device data into the array
  spidata[offset + 1] = opcode;
  spidata[offset] = data;

  SPI.beginTransaction(SPISettings(4000000, MSBFIRST, SPI_MODE0));
  //enable the line 
  digitalWrite(csPin, LOW);

  for(uint8_t i = maxbytes; i > 0; i--)  
    SPI.transfer(spidata[i-1]);
  
  //latch the data onto the display
  digitalWrite(csPin, HIGH);
  SPI.endTransaction();
}
